@@ -35,6 +35,7 @@ declare -a CONFIGURE_ARG_LIST=(
 if [[ "$DISTRO_BRANCH" == -redhat-redhatenterprise*-6.*- ||
       "$DISTRO_BRANCH" == -redhat-centos-6.*- ]]; then
     CONFIGURE_ARG_LIST+=(
+        "--with-smb-idmap-interface-version=5"
         "--disable-cifs-idmap-plugin"
         "--with-syslog=syslog"
         "--without-python3-bindings"
@@ -56,6 +57,30 @@ if [[ "$DISTRO_BRANCH" == -redhat-redhatenterprise*-7.*- ||
     )
 fi
 
+# Different versions of Debian might need different versions here but this is
+# sufficient to make the CI work
+if [[ "$DISTRO_BRANCH" == -debian-* ]]; then
+    CONFIGURE_ARG_LIST+=(
+        "--with-smb-idmap-interface-version=5"
+    )
+fi
+
+if [[ "$DISTRO_BRANCH" == -redhat-fedora-29* ||
+      "$DISTRO_BRANCH" == -redhat-fedora-3* ||
+      "$DISTRO_BRANCH" == -debian-* ||
+      "$DISTRO_BRANCH" == -redhat-redhatenterprise*-8.*- ||
+      "$DISTRO_BRANCH" == -redhat-centos-8.*- ]]; then
+    CONFIGURE_ARG_LIST+=(
+        "--with-crypto=libcrypto"
+    )
+fi
+
+if [[ "$DISTRO_BRANCH" == -redhat-fedora-3[2-9]* ]]; then
+    CONFIGURE_ARG_LIST+=(
+        "--without-python2-bindings"
+    )
+fi
+
 declare -r -a CONFIGURE_ARG_LIST
 
 fi # _CONFIGURE_SH
