@@ -26,8 +26,8 @@ from distutils.core import setup
 
 setup(
     name='SSSDConfig',
-    version='2.2.3',
+    version='2.3.1',
     license='GPLv3+',
-    url='https://pagure.io/SSSD/sssd/',
+    url='https://github.com/SSSD/sssd/',
     packages=['SSSDConfig'],
 )
