@@ -99,10 +99,13 @@ errno_t
 sysdb_save_autofsmap(struct sss_domain_info *domain,
                      const char *name,
                      const char *autofsmapname,
+                     const char *origdn,
                      struct sysdb_attrs *attrs,
                      int cache_timeout,
-                     time_t now)
+                     time_t now,
+                     bool enumerated)
 {
+    time_t expiration = cache_timeout ? now + cache_timeout : 0;
     errno_t ret;
     TALLOC_CTX *tmp_ctx;
 
@@ -136,6 +139,13 @@ sysdb_save_autofsmap(struct sss_domain_info *domain,
         goto done;
     }
 
+    ret = sysdb_attrs_add_string(attrs, SYSDB_ORIG_DN, origdn);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Could not set origdn [%d]: %s\n",
+              ret, strerror(ret));
+        goto done;
+    }
+
     ret = sysdb_attrs_add_string(attrs, SYSDB_NAME, name);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "Could not set name attribute [%d]: %s\n",
@@ -150,15 +160,22 @@ sysdb_save_autofsmap(struct sss_domain_info *domain,
         goto done;
     }
 
-    ret = sysdb_attrs_add_time_t(attrs, SYSDB_CACHE_EXPIRE,
-                                 ((cache_timeout) ?
-                                  (now + cache_timeout) : 0));
+    ret = sysdb_attrs_add_time_t(attrs, SYSDB_CACHE_EXPIRE, expiration);
     if (ret) {
         DEBUG(SSSDBG_OP_FAILURE, "Could not set sysdb cache expire [%d]: %s\n",
               ret, strerror(ret));
         goto done;
     }
 
+    if (enumerated) {
+        ret = sysdb_attrs_add_time_t(attrs, SYSDB_ENUM_EXPIRE, expiration);
+        if (ret) {
+            DEBUG(SSSDBG_OP_FAILURE, "Could not set sysdb enum expire [%d]: %s\n",
+                  ret, strerror(ret));
+            goto done;
+        }
+    }
+
     ret = sysdb_store_custom(domain, name, AUTOFS_MAP_SUBDIR, attrs);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "sysdb_store_custom failed [%d]: %s\n",
@@ -193,6 +210,7 @@ sysdb_get_map_byname(TALLOC_CTX *mem_ctx,
     size_t count;
     struct ldb_message **msgs;
     const char *attrs[] = { SYSDB_OBJECTCLASS,
+                            SYSDB_ORIG_DN,
                             SYSDB_CACHE_EXPIRE,
                             SYSDB_LAST_UPDATE,
                             SYSDB_AUTOFS_MAP_NAME,
@@ -248,7 +266,9 @@ sysdb_save_autofsentry(struct sss_domain_info *domain,
                        const char *map,
                        const char *key,
                        const char *value,
-                       struct sysdb_attrs *attrs)
+                       struct sysdb_attrs *attrs,
+                       int cache_timeout,
+                       time_t now)
 {
     errno_t ret;
     TALLOC_CTX *tmp_ctx;
@@ -307,6 +327,15 @@ sysdb_save_autofsentry(struct sss_domain_info *domain,
         goto done;
     }
 
+    ret = sysdb_attrs_add_time_t(attrs, SYSDB_CACHE_EXPIRE,
+                                 ((cache_timeout) ?
+                                  (now + cache_timeout) : 0));
+    if (ret) {
+        DEBUG(SSSDBG_OP_FAILURE, "Could not set sysdb cache expire [%d]: %s\n",
+              ret, strerror(ret));
+        goto done;
+    }
+
     dn = sysdb_autofsentry_dn(tmp_ctx, domain, map, key, value);
     if (!dn) {
         ret = ENOMEM;
@@ -330,6 +359,79 @@ done:
     return ret;
 }
 
+errno_t
+sysdb_get_autofsentry(TALLOC_CTX *mem_ctx,
+                      struct sss_domain_info *domain,
+                      const char *map_name,
+                      const char *entry_name,
+                      struct ldb_message **_entry)
+{
+    TALLOC_CTX *tmp_ctx;
+    char *safe_entryname;
+    char *filter;
+    struct ldb_dn *mapdn;
+    size_t count;
+    struct ldb_message **msgs;
+    errno_t ret;
+    const char *attrs[] = { SYSDB_AUTOFS_ENTRY_KEY,
+                            SYSDB_AUTOFS_ENTRY_VALUE,
+                            SYSDB_CACHE_EXPIRE,
+                            NULL };
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory!\n");
+        return ENOMEM;
+    }
+
+    ret = sss_filter_sanitize(tmp_ctx, entry_name, &safe_entryname);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Cannot sanitize map [%s] error [%d]: %s\n",
+               map_name, ret, strerror(ret));
+        goto done;
+    }
+
+    filter = talloc_asprintf(tmp_ctx, "(&(objectclass=%s)(%s=%s))",
+                             SYSDB_AUTOFS_ENTRY_OC, SYSDB_AUTOFS_ENTRY_KEY,
+                             safe_entryname);
+    if (filter == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    mapdn = sysdb_autofsmap_dn(tmp_ctx, domain, map_name);
+    if (!mapdn) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_search_entry(tmp_ctx, domain->sysdb, mapdn, LDB_SCOPE_SUBTREE,
+                             filter, attrs, &count, &msgs);
+    if (ret == ENOENT) {
+        goto done;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "sysdb search failed: %d\n", ret);
+        goto done;
+    }
+
+    if (count != 1) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "More than one entry %s:%s found\n",
+              map_name, entry_name);
+        ret = ERR_INTERNAL;
+        goto done;
+    }
+
+    *_entry = talloc_steal(mem_ctx, msgs[0]);
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+
+    return ret;
+}
+
 errno_t
 sysdb_del_autofsentry(struct sss_domain_info *domain,
                       const char *entry_dn)
@@ -347,6 +449,28 @@ sysdb_del_autofsentry(struct sss_domain_info *domain,
     return ret;
 }
 
+errno_t
+sysdb_del_autofsentry_by_key(struct sss_domain_info *domain,
+                             const char *map_name,
+                             const char *entry_key)
+{
+    struct ldb_message *entry;
+    errno_t ret;
+
+    ret = sysdb_get_autofsentry(NULL, domain, map_name, entry_key, &entry);
+    if (ret == ENOENT) {
+        return EOK;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Unable to get autofs entry [%d]: %s\n",
+              ret, sss_strerror(ret));
+        return ret;
+    }
+
+    ret = sysdb_delete_entry(domain->sysdb, entry->dn, true);
+    talloc_free(entry);
+    return ret;
+}
+
 errno_t
 sysdb_autofs_entries_by_map(TALLOC_CTX *mem_ctx,
                             struct sss_domain_info *domain,
@@ -406,6 +530,37 @@ done:
     return ret;
 }
 
+errno_t
+sysdb_set_autofsentry_attr(struct sss_domain_info *domain,
+                           const char *mapname,
+                           const char *key,
+                           const char *value,
+                           struct sysdb_attrs *attrs,
+                           int mod_op)
+{
+    TALLOC_CTX *tmp_ctx;
+    struct ldb_dn *dn;
+    errno_t ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory!\n");
+        return ENOMEM;
+    }
+
+    dn = sysdb_autofsentry_dn(tmp_ctx, domain, mapname, key, value);
+    if (dn == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_set_entry_attr(domain->sysdb, dn, attrs, mod_op);
+
+done:
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 errno_t
 sysdb_set_autofsmap_attr(struct sss_domain_info *domain,
                          const char *name,
@@ -434,6 +589,99 @@ done:
     return ret;
 }
 
+errno_t
+sysdb_invalidate_autofs_entries(struct sss_domain_info *domain,
+                                const char *mapname)
+{
+    TALLOC_CTX *tmp_ctx;
+    bool in_transaction = false;
+    struct ldb_message **entries;
+    struct sysdb_attrs *attrs;
+    const char *value;
+    const char *key;
+    size_t count;
+    errno_t ret;
+    size_t i;
+    int sret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory!\n");
+        return ENOMEM;
+    }
+
+    ret = sysdb_autofs_entries_by_map(tmp_ctx, domain, mapname,
+                                      &count, &entries);
+    if (ret == ENOENT) {
+        ret = EOK;
+        goto done;
+    } else if (ret != EOK) {
+        goto done;
+    }
+
+    attrs = sysdb_new_attrs(tmp_ctx);
+    if (attrs == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_attrs_add_time_t(attrs, SYSDB_CACHE_EXPIRE, 1);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    ret = sysdb_transaction_start(domain->sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to start transaction\n");
+        goto done;
+    }
+    in_transaction = true;
+
+    for (i = 0; i < count; i++) {
+        key = ldb_msg_find_attr_as_string(entries[i], SYSDB_AUTOFS_ENTRY_KEY,
+                                          NULL);
+        if (key == NULL) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "An entry with no key?\n");
+            continue;
+        }
+
+        value = ldb_msg_find_attr_as_string(entries[i],
+                                            SYSDB_AUTOFS_ENTRY_VALUE,
+                                            NULL);
+        if (value == NULL) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "An entry with no value?\n");
+            continue;
+        }
+
+        ret = sysdb_set_autofsentry_attr(domain, mapname, key, value,
+                                         attrs, SYSDB_MOD_REP);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "Could not expire entry %s\n", key);
+            continue;
+        }
+    }
+
+    ret = sysdb_transaction_commit(domain->sysdb);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Could not commit transaction\n");
+        goto done;
+    }
+    in_transaction = false;
+
+    ret = EOK;
+
+done:
+    if (in_transaction) {
+        sret = sysdb_transaction_cancel(domain->sysdb);
+        if (sret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE, "Could not cancel transaction\n");
+        }
+    }
+
+    talloc_free(tmp_ctx);
+    return ret;
+}
+
 errno_t
 sysdb_invalidate_autofs_maps(struct sss_domain_info *domain)
 {
@@ -485,6 +733,11 @@ sysdb_invalidate_autofs_maps(struct sss_domain_info *domain)
         goto done;
     }
 
+    ret = sysdb_attrs_add_time_t(sys_attrs, SYSDB_ENUM_EXPIRE, 1);
+    if (ret != EOK) {
+        goto done;
+    }
+
     ret = sysdb_transaction_start(domain->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to start transaction\n");
@@ -505,6 +758,13 @@ sysdb_invalidate_autofs_maps(struct sss_domain_info *domain)
             DEBUG(SSSDBG_MINOR_FAILURE, "Could not expire map %s\n", name);
             continue;
         }
+
+        ret =  sysdb_invalidate_autofs_entries(domain, name);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_MINOR_FAILURE, "Could not expire map entries %s\n",
+                  name);
+            continue;
+        }
     }
 
     ret = sysdb_transaction_commit(domain->sysdb);
