@@ -31,7 +31,6 @@
 #ifdef HAVE_SYS_INOTIFY_H
 #include <sys/inotify.h>
 #endif
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -406,17 +405,34 @@ static int add_svc_conn_spy(struct mt_svc *svc)
 
 static void svc_child_info(struct mt_svc *svc, int wait_status)
 {
+    int exit_code = 0;
+    int pid = svc->pid;
+    const char *name = (svc->name ? svc->name : "");
+    const char *identity = (svc->identity ? svc->identity : "");
+
     if (WIFEXITED(wait_status)) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              "Child [%d] exited with code [%d]\n",
-               svc->pid, WEXITSTATUS(wait_status));
+        exit_code = WEXITSTATUS(wait_status);
+        if (exit_code == SSS_WATCHDOG_EXIT_CODE) {
+            DEBUG(SSSDBG_CRIT_FAILURE,
+                  "Child [%d] ('%s':'%s') was terminated by own WATCHDOG\n",
+                  pid, name, identity);
+            sss_log(SSS_LOG_CRIT,
+                    "Child [%d] ('%s':'%s') was terminated by own WATCHDOG. "
+                    "Consult corresponding logs to figure out the reason.",
+                    pid, name, identity);
+        } else {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Child [%d] ('%s':'%s') exited with code [%d]\n",
+                   pid, name, identity, exit_code);
+        }
     } else if (WIFSIGNALED(wait_status)) {
         DEBUG(SSSDBG_OP_FAILURE,
-              "Child [%d] terminated with signal [%d]\n",
-               svc->pid, WTERMSIG(wait_status));
+              "Child [%d] ('%s':'%s') terminated with signal [%d]\n",
+               pid, name, identity, WTERMSIG(wait_status));
     } else {
         DEBUG(SSSDBG_FATAL_FAILURE,
-              "Child [%d] did not exit cleanly\n", svc->pid);
+              "Child [%d] ('%s':'%s') did not exit cleanly\n",
+              pid, name, identity);
         /* Forcibly kill this child, just in case */
         kill(svc->pid, SIGKILL);
 
@@ -2224,6 +2240,10 @@ static void service_startup_handler(struct tevent_context *ev,
     DEBUG(SSSDBG_FATAL_FAILURE,
           "Could not exec %s, reason: %s\n", mt_svc->command, strerror(errno));
 
+    /* Also log to syslog since SSSD failed to start. */
+    sss_log(SSS_LOG_ERR, "Could not exec %s, reason: %s",
+            mt_svc->command, strerror(errno));
+
     /* We have to call _exit() instead of exit() here
      * because a bug in D-BUS will cause the server to
      * close its socket at exit() */
@@ -2509,10 +2529,13 @@ int main(int argc, const char *argv[])
     if (opt_genconf == 0) {
         ret = check_file(SSSD_PIDFILE, 0, 0, S_IFREG|0600, 0, NULL, false);
         if (ret == EOK) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                "pidfile exists at %s\n", SSSD_PIDFILE);
-            ERROR("SSSD is already running\n");
-            return 2;
+            ret = check_pidfile(SSSD_PIDFILE);
+            if (ret != EOK) {
+                DEBUG(SSSDBG_FATAL_FAILURE,
+                    "pidfile exists at %s\n", SSSD_PIDFILE);
+                ERROR("SSSD is already running\n");
+                return 2;
+            }
         }
 
         /* Warn if nscd seems to be running */
