@@ -86,7 +86,7 @@ static void authenticate_user(struct tevent_context *ev,
     }
 
     ret = sss_authtok_set_password(pd->authtok, password, keysize);
-    safezero(password, keysize);
+    sss_erase_mem_securely(password, keysize);
     free(password);
     if (ret) {
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -234,6 +234,7 @@ static void delayed_online_authentication_callback(void *private_data)
 }
 
 errno_t add_user_to_delayed_online_authentication(struct krb5_ctx *krb5_ctx,
+                                                  struct sss_domain_info *domain,
                                                   struct pam_data *pd,
                                                   uid_t uid)
 {
@@ -242,6 +243,12 @@ errno_t add_user_to_delayed_online_authentication(struct krb5_ctx *krb5_ctx,
     hash_value_t value;
     struct pam_data *new_pd;
 
+    if (domain->type != DOM_TYPE_POSIX) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              "Domain type does not support delayed authentication\n");
+        return ENOTSUP;
+    }
+
     if (krb5_ctx->deferred_auth_ctx == NULL) {
         DEBUG(SSSDBG_CRIT_FAILURE,
               "Missing context for delayed online authentication.\n");
