@@ -36,18 +36,6 @@ static int lmsg_destructor(void *mem)
     return 0;
 }
 
-static int sdap_msg_attach(TALLOC_CTX *memctx, LDAPMessage *msg)
-{
-    void *h;
-
-    if (!msg) return EINVAL;
-
-    h = sss_mem_attach(memctx, msg, lmsg_destructor);
-    if (!h) return ENOMEM;
-
-    return EOK;
-}
-
 /* ==sdap-handle-utility-functions======================================== */
 
 static inline void sdap_handle_release(struct sdap_handle *sh);
@@ -331,7 +319,7 @@ static void sdap_process_message(struct tevent_context *ev,
         ret = ENOMEM;
     } else {
         reply->msg = msg;
-        ret = sdap_msg_attach(reply, msg);
+        ret = sss_mem_attach(reply, msg, lmsg_destructor);
         if (ret != EOK) {
             ldap_msgfree(msg);
             talloc_zfree(reply);
@@ -481,6 +469,40 @@ int sdap_op_add(TALLOC_CTX *memctx, struct tevent_context *ev,
 
 /* ==Modify-Password====================================================== */
 
+static errno_t
+sdap_chpass_result(TALLOC_CTX *mem_ctx,
+                   int ldap_result,
+                   const char *ldap_msg,
+                   char **_user_msg)
+{
+    errno_t ret;
+
+    switch (ldap_result) {
+    case LDAP_SUCCESS:
+        /* There's no need to set _user_msg here. */
+        return EOK;
+    case LDAP_CONSTRAINT_VIOLATION:
+        if (ldap_msg == NULL || *ldap_msg == '\0') {
+            ldap_msg = "Please make sure the password "
+                       "meets the complexity constraints.";
+        }
+        ret = ERR_CHPASS_DENIED;
+        break;
+    default:
+        ret = ERR_NETWORK_IO;
+    }
+
+    if (ldap_msg != NULL) {
+        *_user_msg = talloc_strdup(mem_ctx, ldap_msg);
+        if (*_user_msg == NULL) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed.\n");
+            return ENOMEM;
+        }
+    }
+
+    return ret;
+}
+
 struct sdap_exop_modify_passwd_state {
     struct sdap_handle *sh;
 
@@ -644,39 +666,7 @@ static void sdap_exop_modify_passwd_done(struct sdap_op *op,
     DEBUG(SSSDBG_MINOR_FAILURE, "ldap_extended_operation result: %s(%d), %s\n",
             sss_ldap_err2string(result), result, errmsg);
 
-    switch (result) {
-    case LDAP_SUCCESS:
-        ret = EOK;
-        break;
-    case LDAP_CONSTRAINT_VIOLATION:
-        if (errmsg && strlen(errmsg) != 0) {
-            state->user_error_message = talloc_strdup(state, errmsg);
-        } else {
-            state->user_error_message = talloc_strdup(state,
-                "Please make sure the password meets the "
-                "complexity constraints.");
-        }
-
-        if (state->user_error_message == NULL) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed\n");
-            ret = ENOMEM;
-            goto done;
-        }
-
-        ret = ERR_CHPASS_DENIED;
-        break;
-    default:
-        if (errmsg) {
-            state->user_error_message = talloc_strdup(state, errmsg);
-            if (state->user_error_message == NULL) {
-                DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed.\n");
-                ret = ENOMEM;
-                goto done;
-            }
-        }
-        ret = ERR_NETWORK_IO;
-        break;
-    }
+    ret = sdap_chpass_result(state, result, errmsg, &state->user_error_message);
 
 done:
     ldap_controls_free(response_controls);
@@ -696,6 +686,7 @@ errno_t sdap_exop_modify_passwd_recv(struct tevent_req *req,
     struct sdap_exop_modify_passwd_state *state = tevent_req_data(req,
                                          struct sdap_exop_modify_passwd_state);
 
+    /* We want to return the error message even on failure */
     *user_error_message = talloc_steal(mem_ctx, state->user_error_message);
 
     TEVENT_REQ_RETURN_ON_ERROR(req);
@@ -703,110 +694,104 @@ errno_t sdap_exop_modify_passwd_recv(struct tevent_req *req,
     return EOK;
 }
 
-/* ==Update-passwordLastChanged-attribute====================== */
-struct update_last_changed_state {
+struct sdap_modify_state {
     struct tevent_context *ev;
     struct sdap_handle *sh;
     struct sdap_op *op;
 
-    const char *dn;
-    LDAPMod **mods;
+    int ldap_result;
+    char *ldap_msg;
 };
 
-static void sdap_modify_shadow_lastchange_done(struct sdap_op *op,
-                                               struct sdap_msg *reply,
-                                               int error, void *pvt);
+static void sdap_modify_done(struct sdap_op *op,
+                             struct sdap_msg *reply,
+                             int error, void *pvt);
 
-struct tevent_req *
-sdap_modify_shadow_lastchange_send(TALLOC_CTX *mem_ctx,
-                                   struct tevent_context *ev,
-                                   struct sdap_handle *sh,
-                                   const char *dn,
-                                   char *lastchanged_name)
+static struct tevent_req *
+sdap_modify_send(TALLOC_CTX *mem_ctx,
+                 struct tevent_context *ev,
+                 struct sdap_handle *sh,
+                 int timeout,
+                 const char *dn,
+                 char *attr,
+                 char **values)
 {
     struct tevent_req *req;
-    struct update_last_changed_state *state;
-    char **values;
+    struct sdap_modify_state *state;
+    LDAPMod **mods;
     errno_t ret;
     int msgid;
 
-    req = tevent_req_create(mem_ctx, &state, struct update_last_changed_state);
+    req = tevent_req_create(mem_ctx, &state, struct sdap_modify_state);
     if (req == NULL) {
         return NULL;
     }
 
     state->ev = ev;
     state->sh = sh;
-    state->dn = dn;
-    state->mods = talloc_zero_array(state, LDAPMod *, 2);
-    if (state->mods == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-    state->mods[0] = talloc_zero(state->mods, LDAPMod);
-    state->mods[1] = talloc_zero(state->mods, LDAPMod);
-    if (!state->mods[0] || !state->mods[1]) {
-        ret = ENOMEM;
-        goto done;
-    }
-    values = talloc_zero_array(state->mods[0], char *, 2);
-    if (values == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-    /* The attribute contains number of days since the epoch */
-    values[0] = talloc_asprintf(values, "%ld", (long)time(NULL)/86400);
-    if (values[0] == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-    state->mods[0]->mod_op = LDAP_MOD_REPLACE;
-    state->mods[0]->mod_type = lastchanged_name;
-    state->mods[0]->mod_vals.modv_strvals = values;
-    state->mods[1] = NULL;
 
-    ret = ldap_modify_ext(state->sh->ldap, state->dn, state->mods,
-            NULL, NULL, &msgid);
-    if (ret) {
+    mods = talloc_zero_array(state, LDAPMod *, 2);
+    if (mods == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    mods[0] = talloc_zero(mods, LDAPMod);
+    if (mods[0] == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    mods[0]->mod_op = LDAP_MOD_REPLACE;
+    mods[0]->mod_type = attr;
+    mods[0]->mod_vals.modv_strvals = values;
+    mods[1] = NULL;
+
+    ret = ldap_modify_ext(state->sh->ldap, dn, mods, NULL, NULL, &msgid);
+    if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to send operation!\n");
         goto done;
     }
 
     ret = sdap_op_add(state, state->ev, state->sh, msgid,
-            sdap_modify_shadow_lastchange_done, req, 5, &state->op);
+                      sdap_modify_done, req, timeout, &state->op);
     if (ret) {
         DEBUG(SSSDBG_CRIT_FAILURE, "Failed to set up operation!\n");
         goto done;
     }
 
+    ret = EOK;
+
 done:
     if (ret != EOK) {
         tevent_req_error(req, ret);
         tevent_req_post(req, ev);
     }
+
     return req;
 }
 
-static void sdap_modify_shadow_lastchange_done(struct sdap_op *op,
-                                               struct sdap_msg *reply,
-                                               int error, void *pvt)
+static void sdap_modify_done(struct sdap_op *op,
+                             struct sdap_msg *reply,
+                             int error, void *pvt)
 {
-    struct tevent_req *req = talloc_get_type(pvt, struct tevent_req);
-    struct update_last_changed_state *state;
-    state = tevent_req_data(req, struct update_last_changed_state);
+    struct tevent_req *req;
+    struct sdap_modify_state *state;
     char *errmsg;
+    errno_t ret;
     int result;
-    errno_t ret = EOK;
     int lret;
 
+    req = talloc_get_type(pvt, struct tevent_req);
+    state = tevent_req_data(req, struct sdap_modify_state);
+
     if (error) {
         tevent_req_error(req, error);
         return;
     }
 
-    lret = ldap_parse_result(state->sh->ldap, reply->msg,
-                            &result, NULL, &errmsg, NULL,
-                            NULL, 0);
+    lret = ldap_parse_result(state->sh->ldap, reply->msg, &result,
+                             NULL, &errmsg, NULL, NULL, 0);
     if (lret != LDAP_SUCCESS) {
         DEBUG(SSSDBG_OP_FAILURE, "ldap_parse_result failed (%d)\n",
                                   state->op->msgid);
@@ -814,10 +799,21 @@ static void sdap_modify_shadow_lastchange_done(struct sdap_op *op,
         goto done;
     }
 
-    DEBUG(SSSDBG_TRACE_LIBS, "Updating lastPwdChange result: %s(%d), %s\n",
+    DEBUG(SSSDBG_TRACE_LIBS, "ldap_modify result: %s(%d), %s\n",
                               sss_ldap_err2string(result),
                               result, errmsg);
 
+    state->ldap_result = result;
+    if (errmsg != NULL) {
+        state->ldap_msg = talloc_strdup(state, errmsg);
+        if (state->ldap_msg == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+    }
+
+    ret = EOK;
+
 done:
     ldap_memfree(errmsg);
 
@@ -828,6 +824,220 @@ done:
     }
 }
 
+static errno_t sdap_modify_recv(TALLOC_CTX *mem_ctx,
+                                struct tevent_req *req,
+                                int *_ldap_result,
+                                char **_ldap_msg)
+{
+    struct sdap_modify_state *state;
+
+    state = tevent_req_data(req, struct sdap_modify_state);
+
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    if (_ldap_result != NULL) {
+        *_ldap_result = state->ldap_result;
+    }
+
+    if (_ldap_msg != NULL) {
+        *_ldap_msg = talloc_steal(mem_ctx, state->ldap_msg);
+    }
+
+    return EOK;
+}
+
+struct sdap_modify_passwd_state {
+    const char *dn;
+    char *user_msg;
+};
+
+static void sdap_modify_passwd_done(struct tevent_req *subreq);
+
+struct tevent_req *
+sdap_modify_passwd_send(TALLOC_CTX *mem_ctx,
+                        struct tevent_context *ev,
+                        struct sdap_handle *sh,
+                        int timeout,
+                        char *attr,
+                        const char *user_dn,
+                        const char *new_password)
+{
+    struct tevent_req *req;
+    struct tevent_req *subreq;
+    struct sdap_modify_passwd_state *state;
+    char **values;
+    errno_t ret;
+
+    req = tevent_req_create(mem_ctx, &state, struct sdap_modify_passwd_state);
+    if (req == NULL) {
+        return NULL;
+    }
+
+    state->dn = user_dn;
+
+    values = talloc_zero_array(state, char *, 2);
+    if (values == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    values[0] = talloc_strdup(values, new_password);
+    if (values[0] == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    subreq = sdap_modify_send(state, ev, sh, timeout, user_dn, attr, values);
+    if (subreq == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    tevent_req_set_callback(subreq, sdap_modify_passwd_done, req);
+
+    ret = EOK;
+
+done:
+    if (ret != EOK) {
+        tevent_req_error(req, ret);
+        tevent_req_post(req, ev);
+    }
+
+    return req;
+}
+
+static void sdap_modify_passwd_done(struct tevent_req *subreq)
+{
+    struct tevent_req *req;
+    struct sdap_modify_passwd_state *state;
+    int ldap_result;
+    char *ldap_msg;
+    errno_t ret;
+
+    req = tevent_req_callback_data(subreq, struct tevent_req);
+    state = tevent_req_data(req, struct sdap_modify_passwd_state);
+
+    ret = sdap_modify_recv(state, subreq, &ldap_result, &ldap_msg);
+    talloc_zfree(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Password change for [%s] failed [%d]: %s\n",
+              state->dn, ret, sss_strerror(ret));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    ret = sdap_chpass_result(state, ldap_result, ldap_msg, &state->user_msg);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Password change for [%s] failed [%d]: %s\n",
+              state->dn, ret, sss_strerror(ret));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC, "Password change for [%s] was successful\n",
+          state->dn);
+
+    tevent_req_done(req);
+}
+
+errno_t sdap_modify_passwd_recv(struct tevent_req *req,
+                                TALLOC_CTX * mem_ctx,
+                                char **_user_error_message)
+{
+    struct sdap_modify_passwd_state *state;
+
+    state = tevent_req_data(req, struct sdap_modify_passwd_state);
+
+    /* We want to return the error message even on failure */
+    *_user_error_message = talloc_steal(mem_ctx, state->user_msg);
+
+    TEVENT_REQ_RETURN_ON_ERROR(req);
+
+    return EOK;
+}
+
+/* ==Update-passwordLastChanged-attribute====================== */
+struct sdap_modify_shadow_lastchange_state {
+    const char *dn;
+};
+
+static void sdap_modify_shadow_lastchange_done(struct tevent_req *subreq);
+
+struct tevent_req *
+sdap_modify_shadow_lastchange_send(TALLOC_CTX *mem_ctx,
+                                   struct tevent_context *ev,
+                                   struct sdap_handle *sh,
+                                   const char *dn,
+                                   char *attr)
+{
+    struct tevent_req *req;
+    struct tevent_req *subreq;
+    struct sdap_modify_shadow_lastchange_state *state;
+    char **values;
+    errno_t ret;
+
+    req = tevent_req_create(mem_ctx, &state,
+                            struct sdap_modify_shadow_lastchange_state);
+    if (req == NULL) {
+        return NULL;
+    }
+
+    state->dn = dn;
+    values = talloc_zero_array(state, char *, 2);
+    if (values == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    /* The attribute contains number of days since the epoch */
+    values[0] = talloc_asprintf(values, "%ld", (long)time(NULL)/86400);
+    if (values[0] == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    subreq = sdap_modify_send(state, ev, sh, 5, dn, attr, values);
+    if (subreq == NULL) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    tevent_req_set_callback(subreq, sdap_modify_shadow_lastchange_done, req);
+
+    ret = EOK;
+
+done:
+    if (ret != EOK) {
+        tevent_req_error(req, ret);
+        tevent_req_post(req, ev);
+    }
+    return req;
+}
+
+static void sdap_modify_shadow_lastchange_done(struct tevent_req *subreq)
+{
+    struct tevent_req *req;
+    struct sdap_modify_shadow_lastchange_state *state;
+    errno_t ret;
+
+    req = tevent_req_callback_data(subreq, struct tevent_req);
+    state = tevent_req_data(req, struct sdap_modify_shadow_lastchange_state);
+
+    ret = sdap_modify_recv(state, subreq, NULL, NULL);
+    talloc_zfree(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "shadowLastChange change for [%s] failed [%d]: %s\n",
+              state->dn, ret, sss_strerror(ret));
+        tevent_req_error(req, ret);
+        return;
+    }
+
+    DEBUG(SSSDBG_TRACE_FUNC, "shadowLastChange change for [%s] was successful\n",
+          state->dn);
+
+    tevent_req_done(req);
+}
+
 errno_t sdap_modify_shadow_lastchange_recv(struct tevent_req *req)
 {
     TEVENT_REQ_RETURN_ON_ERROR(req);
@@ -847,7 +1057,6 @@ struct sdap_get_rootdse_state {
 };
 
 static void sdap_get_rootdse_done(struct tevent_req *subreq);
-static void sdap_get_matching_rule_done(struct tevent_req *subreq);
 
 struct tevent_req *sdap_get_rootdse_send(TALLOC_CTX *memctx,
                                          struct tevent_context *ev,
@@ -899,8 +1108,6 @@ struct tevent_req *sdap_get_rootdse_send(TALLOC_CTX *memctx,
 /* This is not a real attribute, it's just there to avoid
  * actually pulling real data down, to save bandwidth
  */
-#define SDAP_MATCHING_RULE_TEST_ATTR "sssmatchingruletest"
-
 static void sdap_get_rootdse_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
@@ -910,8 +1117,6 @@ static void sdap_get_rootdse_done(struct tevent_req *subreq)
     struct sysdb_attrs **results;
     size_t num_results;
     int ret;
-    const char *filter;
-    const char *attrs[] = { SDAP_MATCHING_RULE_TEST_ATTR, NULL };
 
     ret = sdap_get_generic_recv(subreq, state, &num_results, &results);
     talloc_zfree(subreq);
@@ -940,81 +1145,14 @@ static void sdap_get_rootdse_done(struct tevent_req *subreq)
 
     DEBUG(SSSDBG_TRACE_INTERNAL, "Got rootdse\n");
 
-    /* Auto-detect the LDAP matching rule if requested */
-    if ((!dp_opt_get_bool(state->opts->basic,
-                          SDAP_AD_MATCHING_RULE_INITGROUPS))
-            && !dp_opt_get_bool(state->opts->basic,
-                                SDAP_AD_MATCHING_RULE_GROUPS)) {
-        /* This feature is disabled for both groups
-         * and initgroups. Skip the auto-detection
-         * lookup.
-         */
-        DEBUG(SSSDBG_TRACE_INTERNAL,
-              "Skipping auto-detection of match rule\n");
-        tevent_req_done(req);
-        return;
-    }
-
+    /* This feature is disabled for both groups
+     * and initgroups. Skip the auto-detection
+     * lookup.
+     */
     DEBUG(SSSDBG_TRACE_INTERNAL,
-          "Auto-detecting support for match rule\n");
-
-    /* Create a filter using the matching rule. It need not point
-     * at any valid data. We're only going to be looking for the
-     * error code.
-     */
-    filter = "("SDAP_MATCHING_RULE_TEST_ATTR":"
-             SDAP_MATCHING_RULE_IN_CHAIN":=)";
-
-    /* Perform a trivial query with the matching rule in play.
-     * If it returns success, we know it is available. If it
-     * returns EIO, we know it isn't.
-     */
-    subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
-                                   "", LDAP_SCOPE_BASE, filter, attrs, NULL,
-                                   0, dp_opt_get_int(state->opts->basic,
-                                                     SDAP_SEARCH_TIMEOUT),
-                                   false);
-    if (!subreq) {
-        tevent_req_error(req, ENOMEM);
-        return;
-    }
-    tevent_req_set_callback(subreq, sdap_get_matching_rule_done, req);
-}
-
-static void sdap_get_matching_rule_done(struct tevent_req *subreq)
-{
-    errno_t ret;
-    struct tevent_req *req = tevent_req_callback_data(subreq,
-                                                      struct tevent_req);
-    struct sdap_get_rootdse_state *state = tevent_req_data(req,
-                                             struct sdap_get_rootdse_state);
-    size_t num_results;
-    struct sysdb_attrs **results;
-
-    ret = sdap_get_generic_recv(subreq, state, &num_results, &results);
-    talloc_zfree(subreq);
-    if (ret == EOK) {
-        /* The search succeeded */
-        state->opts->support_matching_rule = true;
-    } else if (ret == EIO) {
-        /* The search failed. Disable support for
-         * matching rule lookups.
-         */
-        state->opts->support_matching_rule = false;
-    } else {
-        DEBUG(SSSDBG_MINOR_FAILURE,
-              "Unexpected error while testing for matching rule support\n");
-        tevent_req_error(req, ret);
-        return;
-    }
-
-    DEBUG(SSSDBG_CONF_SETTINGS,
-          "LDAP server %s the matching rule extension\n",
-          state->opts->support_matching_rule
-              ? "supports"
-              : "does not support");
-
+          "Skipping auto-detection of match rule\n");
     tevent_req_done(req);
+    return;
 }
 
 int sdap_get_rootdse_recv(struct tevent_req *req,
@@ -1297,7 +1435,7 @@ sdap_get_generic_ext_send(TALLOC_CTX *memctx,
     state->serverctrls[i] = NULL;
 
     PROBE(SDAP_GET_GENERIC_EXT_SEND, state->search_base,
-          state->scope, state->filter);
+          state->scope, state->filter, state->attrs);
 
     ret = sdap_get_generic_ext_step(req);
     if (ret != EOK) {
@@ -2787,7 +2925,9 @@ int sdap_deref_search_recv(struct tevent_req *req,
     return EOK;
 }
 
-bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts)
+bool sdap_has_deref_support_ex(struct sdap_handle *sh,
+                               struct sdap_options *opts,
+                               bool ignore_client)
 {
     const char *deref_oids[][2] = { { LDAP_SERVER_ASQ_OID, "ASQ" },
                                     { LDAP_CONTROL_X_DEREF, "OpenLDAP" },
@@ -2800,18 +2940,25 @@ bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts)
         return false;
     }
 
-    deref_threshold = dp_opt_get_int(opts->basic, SDAP_DEREF_THRESHOLD);
-    if (deref_threshold == 0) {
-        return false;
+    if (ignore_client == false) {
+        deref_threshold = dp_opt_get_int(opts->basic, SDAP_DEREF_THRESHOLD);
+        if (deref_threshold == 0) {
+            return false;
+        }
     }
 
     for (i=0; deref_oids[i][0]; i++) {
         if (sdap_is_control_supported(sh, deref_oids[i][0])) {
             DEBUG(SSSDBG_TRACE_FUNC, "The server supports deref method %s\n",
-                      deref_oids[i][1]);
+                  deref_oids[i][1]);
             return true;
         }
     }
 
     return false;
 }
+
+bool sdap_has_deref_support(struct sdap_handle *sh, struct sdap_options *opts)
+{
+    return sdap_has_deref_support_ex(sh, opts, false);
+}
