@@ -102,6 +102,42 @@ static errno_t proxy_auth_conf(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
+static errno_t proxy_resolver_conf(TALLOC_CTX *mem_ctx,
+                                   struct be_ctx *be_ctx,
+                                   char **_libname)
+{
+    TALLOC_CTX *tmp_ctx;
+    char *libname;
+    errno_t ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (tmp_ctx == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_new() failed\n");
+        return ENOMEM;
+    }
+
+    ret = confdb_get_string(be_ctx->cdb, tmp_ctx, be_ctx->conf_path,
+                            CONFDB_PROXY_RESOLVER_LIBNAME, NULL, &libname);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to read confdb [%d]: %s\n",
+              ret, sss_strerror(ret));
+        goto done;
+    } else if (libname == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "No resolver library name given\n");
+        ret = ENOENT;
+        goto done;
+    }
+
+    *_libname = talloc_steal(mem_ctx, libname);
+
+    ret = EOK;
+
+done:
+    talloc_free(tmp_ctx);
+
+    return ret;
+}
+
 static errno_t proxy_init_auth_ctx(TALLOC_CTX *mem_ctx,
                                    struct be_ctx *be_ctx,
                                    struct data_provider *provider,
@@ -176,112 +212,135 @@ errno_t sssm_proxy_init(TALLOC_CTX *mem_ctx,
                        const char *module_name,
                        void **_module_data)
 {
-    struct proxy_auth_ctx *auth_ctx;
+    struct proxy_module_ctx *module_ctx;
     errno_t ret;
 
-    if (!dp_target_enabled(provider, module_name,
-                           DPT_ACCESS, DPT_AUTH, DPT_CHPASS)) {
-        return EOK;
+    module_ctx = talloc_zero(mem_ctx, struct proxy_module_ctx);
+    if (module_ctx == NULL) {
+        return ENOMEM;
     }
 
-    /* Initialize auth_ctx since one of the access, auth or chpass is set. */
-
-    ret = proxy_init_auth_ctx(mem_ctx, be_ctx, provider, &auth_ctx);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Unable to create auth context [%d]: %s\n",
-              ret, sss_strerror(ret));
-        return ret;
+    if (dp_target_enabled(provider, module_name,
+                          DPT_ACCESS, DPT_AUTH, DPT_CHPASS)) {
+        /* Initialize auth_ctx since one of the access, auth or chpass is
+         * set. */
+        ret = proxy_init_auth_ctx(module_ctx, be_ctx, provider,
+                                  &module_ctx->auth_ctx);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_CRIT_FAILURE, "Unable to create auth context [%d]: %s\n",
+                  ret, sss_strerror(ret));
+            talloc_free(module_ctx);
+            return ret;
+        }
     }
 
-    *_module_data = auth_ctx;
+    *_module_data = module_ctx;
 
     return EOK;
 }
 
-
-#define ERROR_INITGR "The '%s' library does not provides the " \
-                         "_nss_XXX_initgroups_dyn function!\n" \
-                         "initgroups will be slow as it will require " \
-                         "full groups enumeration!\n"
-#define ERROR_NETGR "The '%s' library does not support netgroups.\n"
-#define ERROR_SERV "The '%s' library does not support services.\n"
-
 static errno_t proxy_load_nss_symbols(struct sss_nss_ops *ops,
                                       const char *libname)
 {
     errno_t ret;
-    size_t i;
+    struct sss_nss_symbols syms[] = {
+        {(void*)&ops->getpwnam_r,      true,  "getpwnam_r" },
+        {(void*)&ops->getpwuid_r,      true,  "getpwuid_r" },
+        {(void*)&ops->setpwent,        true,  "setpwent" },
+        {(void*)&ops->getpwent_r,      true,  "getpwent_r" },
+        {(void*)&ops->endpwent,        true,  "endpwent" },
+        {(void*)&ops->getgrnam_r,      true,  "getgrnam_r" },
+        {(void*)&ops->getgrgid_r,      true,  "getgrgid_r" },
+        {(void*)&ops->setgrent,        true,  "setgrent" },
+        {(void*)&ops->getgrent_r,      true,  "getgrent_r" },
+        {(void*)&ops->endgrent,        true,  "endgrent" },
+        {(void*)&ops->initgroups_dyn,  false, "initgroups_dyn" },
+        {(void*)&ops->setnetgrent,     false, "setnetgrent" },
+        {(void*)&ops->getnetgrent_r,   false, "getnetgrent_r" },
+        {(void*)&ops->endnetgrent,     false, "endnetgrent" },
+        {(void*)&ops->getservbyname_r, false, "getservbyname_r" },
+        {(void*)&ops->getservbyport_r, false, "getservbyport_r" },
+        {(void*)&ops->setservent,      false, "setservent" },
+        {(void*)&ops->getservent_r,    false, "getservent_r" },
+        {(void*)&ops->endservent,      false, "endservent" },
+    };
+    size_t nsyms = sizeof(syms) / sizeof(struct sss_nss_symbols);
 
-    ret = sss_load_nss_symbols(ops, libname);
+    ret = sss_load_nss_symbols(ops, libname, syms, nsyms);
     if (ret != EOK) {
         return ret;
     }
 
-    struct {
-        void *fptr;
-        const char* custom_error;
-    } optional_syms[] = {
-        {(void*)ops->initgroups_dyn,  ERROR_INITGR},
-        {(void*)ops->setnetgrent,     ERROR_NETGR},
-        {(void*)ops->getnetgrent_r,   ERROR_NETGR},
-        {(void*)ops->endnetgrent,     ERROR_NETGR},
-        {(void*)ops->getservbyname_r, ERROR_SERV},
-        {(void*)ops->getservbyport_r, ERROR_SERV},
-        {(void*)ops->setservent,      ERROR_SERV},
-        {(void*)ops->getservent_r,    ERROR_SERV},
-        {(void*)ops->endservent,      ERROR_SERV},
-    };
-    for (i = 0; i < sizeof(optional_syms) / sizeof(optional_syms[0]); ++i) {
-        if (!optional_syms[i].fptr) {
-            DEBUG(SSSDBG_CRIT_FAILURE, optional_syms[i].custom_error, libname);
-        }
-    }
-
-    void *mandatory_syms[] = {
-        (void*)ops->getpwnam_r,
-        (void*)ops->getpwuid_r,
-        (void*)ops->setpwent,
-        (void*)ops->getpwent_r,
-        (void*)ops->endpwent,
-        (void*)ops->getgrnam_r,
-        (void*)ops->getgrgid_r,
-        (void*)ops->setgrent,
-        (void*)ops->getgrent_r,
-        (void*)ops->endgrent,
-    };
-    for (i = 0; i < sizeof(mandatory_syms)/sizeof(mandatory_syms[0]); ++i) {
-        if (!mandatory_syms[i]) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "The '%s' library does not provide mandatory function", libname);
-            return ELIBBAD;
-        }
-    }
-
     return EOK;
 }
 
+static errno_t proxy_load_nss_hosts_symbols(struct sss_nss_ops *ops,
+                                            const char *libname)
+{
+    errno_t ret;
+    struct sss_nss_symbols syms[] = {
+        {(void*)&ops->gethostbyname_r,  true,  "gethostbyname_r"},
+        {(void*)&ops->gethostbyname2_r, true,  "gethostbyname2_r"},
+        {(void*)&ops->gethostbyaddr_r,  true,  "gethostbyaddr_r"},
+        {(void*)&ops->sethostent,       false, "sethostent"},
+        {(void*)&ops->gethostent_r,     false, "gethostent_r"},
+        {(void*)&ops->endhostent,       false, "endhostent"},
+    };
+    size_t nsyms = sizeof(syms) / sizeof(struct sss_nss_symbols);
+
+    ret = sss_load_nss_symbols(ops, libname, syms, nsyms);
+    if (ret != EOK) {
+        return ret;
+    }
+
+    return EOK;
+}
+
+static errno_t proxy_load_nss_nets_symbols(struct sss_nss_ops *ops,
+                                           const char *libname)
+{
+    errno_t ret;
+    struct sss_nss_symbols syms[] = {
+        {(void*)&ops->getnetbyname_r,  true,  "getnetbyname_r"},
+        {(void*)&ops->getnetbyaddr_r,  true,  "getnetbyaddr_r"},
+        {(void*)&ops->setnetent,       false, "setnetent"},
+        {(void*)&ops->getnetent_r,     false, "getnetent_r"},
+        {(void*)&ops->endnetent,       false, "endnetent"},
+    };
+    size_t nsyms = sizeof(syms) / sizeof(struct sss_nss_symbols);
+
+    ret = sss_load_nss_symbols(ops, libname, syms, nsyms);
+    if (ret != EOK) {
+        return ret;
+    }
+
+    return EOK;
+}
 
 errno_t sssm_proxy_id_init(TALLOC_CTX *mem_ctx,
                            struct be_ctx *be_ctx,
                            void *module_data,
                            struct dp_method *dp_methods)
 {
-    struct proxy_id_ctx *ctx;
+    struct proxy_module_ctx *module_ctx;
     char *libname;
     errno_t ret;
 
-    ctx = talloc_zero(mem_ctx, struct proxy_id_ctx);
-    if (ctx == NULL) {
+    module_ctx = talloc_get_type(module_data, struct proxy_module_ctx);
+    module_ctx->id_ctx = talloc_zero(module_ctx, struct proxy_id_ctx);
+    if (module_ctx->id_ctx == NULL) {
         return ENOMEM;
     }
 
-    ctx->be = be_ctx;
+    module_ctx->id_ctx->be = be_ctx;
 
-    ret = proxy_id_conf(ctx, be_ctx, &libname, &ctx->fast_alias);
+    ret = proxy_id_conf(module_ctx->id_ctx, be_ctx, &libname,
+                        &module_ctx->id_ctx->fast_alias);
     if (ret != EOK) {
         goto done;
     }
 
-    ret = proxy_load_nss_symbols(&ctx->ops, libname);
+    ret = proxy_load_nss_symbols(&module_ctx->id_ctx->ops, libname);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE, "Unable to load NSS symbols [%d]: %s\n",
               ret, sss_strerror(ret));
@@ -289,8 +348,9 @@ errno_t sssm_proxy_id_init(TALLOC_CTX *mem_ctx,
     }
 
     dp_set_method(dp_methods, DPM_ACCOUNT_HANDLER,
-                  proxy_account_info_handler_send, proxy_account_info_handler_recv, ctx,
-                  struct proxy_id_ctx, struct dp_id_data, struct dp_reply_std);
+                  proxy_account_info_handler_send, proxy_account_info_handler_recv,
+                  module_ctx->id_ctx, struct proxy_id_ctx, struct dp_id_data,
+                  struct dp_reply_std);
 
     dp_set_method(dp_methods, DPM_ACCT_DOMAIN_HANDLER,
                   default_account_domain_send, default_account_domain_recv, NULL,
@@ -300,7 +360,7 @@ errno_t sssm_proxy_id_init(TALLOC_CTX *mem_ctx,
 
 done:
     if (ret != EOK) {
-        talloc_free(ctx);
+        talloc_zfree(module_ctx->id_ctx);
     }
 
     return ret;
@@ -311,13 +371,14 @@ errno_t sssm_proxy_auth_init(TALLOC_CTX *mem_ctx,
                              void *module_data,
                              struct dp_method *dp_methods)
 {
-    struct proxy_auth_ctx *auth_ctx;
+    struct proxy_module_ctx *module_ctx;
 
-    auth_ctx = talloc_get_type(module_data, struct proxy_auth_ctx);
+    module_ctx = talloc_get_type(module_data, struct proxy_module_ctx);
 
     dp_set_method(dp_methods, DPM_AUTH_HANDLER,
-                  proxy_pam_handler_send, proxy_pam_handler_recv, auth_ctx,
-                  struct proxy_auth_ctx, struct pam_data, struct pam_data *);
+                  proxy_pam_handler_send, proxy_pam_handler_recv,
+                  module_ctx->auth_ctx, struct proxy_auth_ctx,
+                  struct pam_data, struct pam_data *);
 
     return EOK;
 }
@@ -335,13 +396,72 @@ errno_t sssm_proxy_access_init(TALLOC_CTX *mem_ctx,
                                void *module_data,
                                struct dp_method *dp_methods)
 {
-    struct proxy_auth_ctx *auth_ctx;
+    struct proxy_module_ctx *module_ctx;
 
-    auth_ctx = talloc_get_type(module_data, struct proxy_auth_ctx);
+    module_ctx = talloc_get_type(module_data, struct proxy_module_ctx);
 
     dp_set_method(dp_methods, DPM_ACCESS_HANDLER,
-                  proxy_pam_handler_send, proxy_pam_handler_recv, auth_ctx,
-                  struct proxy_auth_ctx, struct pam_data, struct pam_data *);
+                  proxy_pam_handler_send, proxy_pam_handler_recv,
+                  module_ctx->auth_ctx, struct proxy_auth_ctx,
+                  struct pam_data, struct pam_data *);
 
     return EOK;
 }
+
+errno_t sssm_proxy_resolver_init(TALLOC_CTX *mem_ctx,
+                                 struct be_ctx *be_ctx,
+                                 void *module_data,
+                                 struct dp_method *dp_methods)
+{
+    struct proxy_module_ctx *module_ctx;
+    char *libname;
+    errno_t ret;
+
+    module_ctx = talloc_get_type(module_data, struct proxy_module_ctx);
+
+    module_ctx->resolver_ctx = talloc_zero(mem_ctx, struct proxy_resolver_ctx);
+    if (module_ctx->resolver_ctx == NULL) {
+        return ENOMEM;
+    }
+
+    ret = proxy_resolver_conf(module_ctx->resolver_ctx, be_ctx, &libname);
+    if (ret == ENOENT) {
+        ret = ENOTSUP;
+        goto done;
+    } else if (ret != EOK) {
+        goto done;
+    }
+
+    ret = proxy_load_nss_hosts_symbols(&module_ctx->resolver_ctx->ops, libname);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to load NSS symbols [%d]: %s\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+    ret = proxy_load_nss_nets_symbols(&module_ctx->resolver_ctx->ops, libname);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Unable to load NSS symbols [%d]: %s\n",
+              ret, sss_strerror(ret));
+        goto done;
+    }
+
+    dp_set_method(dp_methods, DPM_RESOLVER_HOSTS_HANDLER,
+                  proxy_hosts_handler_send, proxy_hosts_handler_recv,
+                  module_ctx->resolver_ctx, struct proxy_resolver_ctx,
+                  struct dp_resolver_data, struct dp_reply_std);
+
+    dp_set_method(dp_methods, DPM_RESOLVER_IP_NETWORK_HANDLER,
+                  proxy_nets_handler_send, proxy_nets_handler_recv,
+                  module_ctx->resolver_ctx, struct proxy_resolver_ctx,
+                  struct dp_resolver_data, struct dp_reply_std);
+
+    ret = EOK;
+
+done:
+    if (ret != EOK) {
+        talloc_zfree(module_ctx->resolver_ctx);
+    }
+
+    return ret;
+}
