@@ -46,8 +46,6 @@ cache_req_data_create_attrs(TALLOC_CTX *mem_ctx,
         return NULL;
     }
 
-    i = 0;
-
     for (i = 0; i < reqnum; i++) {
         attrs[i] = talloc_strdup(attrs, requested[i]);
         if (attrs[i] == NULL) {
@@ -56,7 +54,7 @@ cache_req_data_create_attrs(TALLOC_CTX *mem_ctx,
         }
     }
 
-    for (; i < total; i++) {
+    for (/* continue */; i < total; i++) {
         attrs[i] = talloc_strdup(attrs, defattrs[i - reqnum]);
         if (attrs[i] == NULL) {
             talloc_free(attrs);
@@ -96,6 +94,8 @@ cache_req_data_create(TALLOC_CTX *mem_ctx,
     case CACHE_REQ_OBJECT_BY_NAME:
     case CACHE_REQ_AUTOFS_MAP_ENTRIES:
     case CACHE_REQ_AUTOFS_MAP_BY_NAME:
+    case CACHE_REQ_IP_HOST_BY_NAME:
+    case CACHE_REQ_IP_NETWORK_BY_NAME:
         if (input->name.input == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Bug: name cannot be NULL!\n");
             ret = ERR_INTERNAL;
@@ -108,6 +108,17 @@ cache_req_data_create(TALLOC_CTX *mem_ctx,
             goto done;
         }
         break;
+    case CACHE_REQ_IP_HOST_BY_ADDR:
+    case CACHE_REQ_IP_NETWORK_BY_ADDR:
+        data->addr.af = input->addr.af;
+        data->addr.len = input->addr.len;
+        data->addr.data = talloc_memdup(data, input->addr.data,
+                                        input->addr.len);
+        if (data->addr.data == NULL) {
+            ret = ENOMEM;
+            goto done;
+        }
+        break;
     case CACHE_REQ_USER_BY_CERT:
         if (input->cert == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Bug: certificate cannot be NULL!\n");
@@ -142,6 +153,8 @@ cache_req_data_create(TALLOC_CTX *mem_ctx,
     case CACHE_REQ_ENUM_USERS:
     case CACHE_REQ_ENUM_GROUPS:
     case CACHE_REQ_ENUM_SVC:
+    case CACHE_REQ_ENUM_HOST:
+    case CACHE_REQ_ENUM_IP_NETWORK:
         break;
     case CACHE_REQ_SVC_BY_NAME:
         if (input->svc.name->input == NULL) {
@@ -187,7 +200,7 @@ cache_req_data_create(TALLOC_CTX *mem_ctx,
         }
 
         break;
-    case CACHE_REQ_HOST_BY_NAME:
+    case CACHE_REQ_SSH_HOST_ID_BY_NAME:
         if (input->name.input == NULL) {
             DEBUG(SSSDBG_CRIT_FAILURE, "Bug: name cannot be NULL!\n");
             ret = ERR_INTERNAL;
@@ -361,11 +374,11 @@ cache_req_data_svc(TALLOC_CTX *mem_ctx,
 }
 
 struct cache_req_data *
-cache_req_data_host(TALLOC_CTX *mem_ctx,
-                    enum cache_req_type type,
-                    const char *name,
-                    const char *alias,
-                    const char **attrs)
+cache_req_data_ssh_host_id(TALLOC_CTX *mem_ctx,
+                           enum cache_req_type type,
+                           const char *name,
+                           const char *alias,
+                           const char **attrs)
 {
     struct cache_req_data input = {0};
 
@@ -376,6 +389,22 @@ cache_req_data_host(TALLOC_CTX *mem_ctx,
     return cache_req_data_create(mem_ctx, type, &input);
 }
 
+struct cache_req_data *
+cache_req_data_addr(TALLOC_CTX *mem_ctx,
+                    enum cache_req_type type,
+                    uint32_t af,
+                    uint32_t addrlen,
+                    uint8_t *addr)
+{
+    struct cache_req_data input = {0};
+
+    input.addr.af = af;
+    input.addr.len = addrlen;
+    input.addr.data = addr;
+
+    return cache_req_data_create(mem_ctx, type, &input);
+}
+
 struct cache_req_data *
 cache_req_data_autofs_entry(TALLOC_CTX *mem_ctx,
                             enum cache_req_type type,
