@@ -32,6 +32,13 @@ struct pam_auth_req;
 
 typedef void (pam_dp_callback_t)(struct pam_auth_req *preq);
 
+enum pam_initgroups_scheme {
+    PAM_INITGR_NEVER,
+    PAM_INITGR_NO_SESSION,
+    PAM_INITGR_ALWAYS,
+    PAM_INITGR_INVALID
+};
+
 struct pam_ctx {
     struct resp_ctx *rctx;
     time_t id_timeout;
@@ -47,13 +54,14 @@ struct pam_ctx {
     char **app_services;
 
     bool cert_auth;
-    int p11_child_debug_fd;
     char *nss_db;
     struct sss_certmap_ctx *sss_certmap_ctx;
     char **smartcard_services;
 
     char **prompting_config_sections;
     int num_prompting_config_sections;
+
+    enum pam_initgroups_scheme initgroups_scheme;
 };
 
 struct pam_auth_req {
@@ -101,7 +109,6 @@ void sss_cai_check_users(struct cert_auth_info **list, size_t *_cert_count,
 
 struct tevent_req *pam_check_cert_send(TALLOC_CTX *mem_ctx,
                                        struct tevent_context *ev,
-                                       int child_debug_fd,
                                        const char *nss_db,
                                        time_t timeout,
                                        const char *verify_opts,
@@ -132,4 +139,6 @@ errno_t filter_responses(struct confdb_ctx *cdb,
 
 errno_t pam_eval_prompting_config(struct pam_ctx *pctx, struct pam_data *pd);
 
+enum pam_initgroups_scheme pam_initgroups_string_to_enum(const char *str);
+const char *pam_initgroup_enum_to_string(enum pam_initgroups_scheme scheme);
 #endif /* __PAMSRV_H__ */
