@@ -36,10 +36,10 @@ struct ssh_ctx {
     char *ca_db;
     bool use_cert_keys;
 
-    int p11_child_debug_fd;
     time_t certmap_last_read;
     struct sss_certmap_ctx *sss_certmap_ctx;
     char **cert_rules;
+    bool cert_rules_error;
 };
 
 struct sss_cmd_table *get_ssh_cmds(void);
