@@ -4,14 +4,15 @@
 # Copyright (c) 2017 Red Hat, Inc.
 # Author: Lukas Slebodnik <lslebodn@redhat.com>
 #
-# This is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 only
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -203,15 +204,17 @@ def format_basic_conf(ldap_conn, schema):
 
         [ifp]
         # it need to be executed with valgrind because there is a problem
-        # problem with "ifp" + client regristration in monitor
+        # problem with "ifp" + client registration in monitor
         # There is not such problem in 1st test. Just in following tests.
         command = {ifp_command} --uid 0 --gid 0 --debug-to-files
+        user_attributes = +extraName
 
         [domain/LDAP]
         {schema_conf}
         id_provider         = ldap
         ldap_uri            = {ldap_conn.ds_inst.ldap_url}
         ldap_search_base    = {ldap_conn.ds_inst.base_dn}
+        ldap_user_extra_attrs = extraName:uid
 
         [application/app]
         inherit_from = LDAP
@@ -345,10 +348,8 @@ def test_ping_raw(dbus_system_bus, ldap_conn, simple_rfc2307):
     assert exc_info.errisinstance(dbus.exceptions.DBusException)
 
     ex = exc_info.value
-    assert ex.get_dbus_name() == 'org.freedesktop.DBus.Error.InvalidArgs'
-    assert ex.get_dbus_message() == 'Argument 0 is specified to be of type ' \
-                                    '"string", but is actually of type ' \
-                                    '"invalid"\n'
+    assert ex.get_dbus_name() == 'sbus.Error.Errno'
+    assert 'Unexpected argument type provided' in ex.get_dbus_message()
 
     # test wrong parameter type
     with pytest.raises(dbus.exceptions.DBusException) as exc_info:
@@ -356,10 +357,8 @@ def test_ping_raw(dbus_system_bus, ldap_conn, simple_rfc2307):
     assert exc_info.errisinstance(dbus.exceptions.DBusException)
 
     ex = exc_info.value
-    assert ex.get_dbus_name() == 'org.freedesktop.DBus.Error.InvalidArgs'
-    assert ex.get_dbus_message() == 'Argument 0 is specified to be of type ' \
-                                    '"string", but is actually of type ' \
-                                    '"int32"\n'
+    assert ex.get_dbus_name() == 'sbus.Error.Errno'
+    assert 'Unexpected argument type provided' in ex.get_dbus_message()
 
     # test wrong parameter value
     with pytest.raises(dbus.exceptions.DBusException) as exc_info:
@@ -368,7 +367,7 @@ def test_ping_raw(dbus_system_bus, ldap_conn, simple_rfc2307):
 
     ex = exc_info.value
     assert ex.get_dbus_name() == 'org.freedesktop.DBus.Error.InvalidArgs'
-    assert ex.get_dbus_message() == 'Ping() only accepts "ping" as a param\n'
+    assert ex.get_dbus_message() == 'Invalid argument'
 
     # positive test
     ret = sssd_interface.Ping('ping')
@@ -411,7 +410,7 @@ def test_ping_introspection(dbus_system_bus, ldap_conn, simple_rfc2307):
 
     ex = exc_info.value
     assert ex.get_dbus_name() == 'org.freedesktop.DBus.Error.InvalidArgs'
-    assert ex.get_dbus_message() == 'Ping() only accepts "ping" as a param\n'
+    assert ex.get_dbus_message() == 'Invalid argument'
 
     # positive test
     ret = sssd_interface.Ping('ping')
@@ -461,8 +460,8 @@ def test_get_user_attr(dbus_system_bus, ldap_conn, sanity_rfc2307):
     assert exc_info.errisinstance(dbus.exceptions.DBusException)
 
     ex = exc_info.value
-    assert ex.get_dbus_name() == 'org.freedesktop.DBus.Error.Failed'
-    assert ex.get_dbus_message() == 'No such user\n'
+    assert ex.get_dbus_name() == 'sbus.Error.NotFound'
+    assert ex.get_dbus_message() == 'No such file or directory'
 
     # test 0 attributes
     user_attrs = sssd_interface.GetUserAttr('user1', [])
@@ -505,8 +504,8 @@ def test_get_user_groups(dbus_system_bus, ldap_conn, sanity_rfc2307):
     assert exc_info.errisinstance(dbus.exceptions.DBusException)
 
     ex = exc_info.value
-    assert ex.get_dbus_name() == 'org.freedesktop.DBus.Error.Failed'
-    assert ex.get_dbus_message() == 'No such user\n'
+    assert ex.get_dbus_name() == 'sbus.Error.NotFound'
+    assert ex.get_dbus_message() == 'No such file or directory'
 
     # the same test via nss responder
     with pytest.raises(KeyError):
@@ -537,6 +536,35 @@ def test_get_user_groups(dbus_system_bus, ldap_conn, sanity_rfc2307):
     assert sorted(res) == ['single_user_group', 'two_user_group']
 
 
+def get_user_property(dbus_system_bus, username, prop_name):
+    users_obj = dbus_system_bus.get_object(
+                                        'org.freedesktop.sssd.infopipe',
+                                        '/org/freedesktop/sssd/infopipe/Users')
+
+    users_iface = dbus.Interface(users_obj,
+                                 "org.freedesktop.sssd.infopipe.Users")
+
+    user_path = users_iface.FindByName(username)
+    user_object = dbus_system_bus.get_object('org.freedesktop.sssd.infopipe',
+                                             user_path)
+
+    prop_iface = dbus.Interface(user_object, 'org.freedesktop.DBus.Properties')
+    return prop_iface.Get('org.freedesktop.sssd.infopipe.Users.User',
+                          prop_name)
+
+
+def test_get_extra_attributes_empty(dbus_system_bus,
+                                    ldap_conn,
+                                    sanity_rfc2307):
+    """
+    Make sure the extraAttributes property can be retrieved
+    """
+    extra_attrs = get_user_property(dbus_system_bus,
+                                    'user1',
+                                    'extraAttributes')
+    assert extra_attrs['extraName'][0] == 'user1'
+
+
 def test_sssctl_domain_list_app_domain(dbus_system_bus,
                                        ldap_conn,
                                        sanity_rfc2307):
