@@ -3,14 +3,15 @@
 #
 # Copyright (c) 2016 Red Hat, Inc.
 #
-# This is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 only
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -42,7 +43,7 @@ def create_conf_fixture(request, contents):
     request.addfinalizer(lambda: os.unlink(config.CONF_PATH))
 
 
-def create_sssd_secrets_fixture(request, teardown=True):
+def create_sssd_secrets_fixture(request, teardown=True, env=None):
     if subprocess.call(['sssd', "--genconf"]) != 0:
         raise Exception("failed to regenerate confdb")
 
@@ -56,7 +57,9 @@ def create_sssd_secrets_fixture(request, teardown=True):
     assert secpid >= 0
 
     if secpid == 0:
-        os.execv(resp_path, ("--uid=0", "--gid=0"))
+        if env is None:
+            env = os.environ
+        os.execve(resp_path, (resp_path, "--uid=0", "--gid=0"), env)
         print("sssd_secrets failed to start")
         sys.exit(99)
     else:
@@ -632,7 +635,7 @@ def setup_for_resp_timeout_test(request):
 @pytest.mark.slow
 def test_resp_idle_timeout_shutdown_slow(setup_for_resp_timeout_test):
     """
-    Test that the responder is shutdown after the respoder_idle_timeout is
+    Test that the responder is shutdown after the responder_idle_timeout is
     over
     """
     secpid = setup_for_resp_timeout_test
