@@ -24,11 +24,10 @@
 
 struct cert_to_ssh_key_state {
     struct tevent_context *ev;
-    int child_debug_fd;
+    const char *logfile;
     time_t timeout;
     const char **extra_args;
     const char **certs;
-    struct ldb_val *bin_certs;
     struct ldb_val *keys;
     size_t cert_count;
     size_t iter;
@@ -46,7 +45,7 @@ static void cert_to_ssh_key_done(int child_status,
 
 struct tevent_req *cert_to_ssh_key_send(TALLOC_CTX *mem_ctx,
                                         struct tevent_context *ev,
-                                        int child_debug_fd, time_t timeout,
+                                        const char *logfile, time_t timeout,
                                         const char *ca_db,
                                         struct sss_certmap_ctx *sss_certmap_ctx,
                                         size_t cert_count,
@@ -71,10 +70,8 @@ struct tevent_req *cert_to_ssh_key_send(TALLOC_CTX *mem_ctx,
     }
 
     state->ev = ev;
-    state->child_debug_fd = (child_debug_fd == -1) ? STDERR_FILENO
-                                                   : child_debug_fd;
+    state->logfile = logfile;
     state->timeout = timeout;
-    state->bin_certs = bin_certs;
     state->io = talloc(state, struct child_io_fds);
     if (state->io == NULL) {
         DEBUG(SSSDBG_OP_FAILURE, "talloc failed.\n");
@@ -138,6 +135,7 @@ struct tevent_req *cert_to_ssh_key_send(TALLOC_CTX *mem_ctx,
             ret = EINVAL;
             goto done;
         }
+
         state->cert_count++;
     }
 
@@ -206,7 +204,7 @@ static errno_t cert_to_ssh_key_step(struct tevent_req *req)
     child_pid = fork();
     if (child_pid == 0) { /* child */
         exec_child_ex(state, pipefd_to_child, pipefd_from_child, P11_CHILD_PATH,
-                      state->child_debug_fd, state->extra_args, false,
+                      state->logfile, state->extra_args, false,
                       STDIN_FILENO, STDOUT_FILENO);
         /* We should never get here */
         DEBUG(SSSDBG_CRIT_FAILURE, "BUG: Could not exec p11 child\n");
@@ -289,11 +287,10 @@ static void cert_to_ssh_key_done(int child_status,
     if (valid) {
         DEBUG(SSSDBG_TRACE_LIBS, "Certificate [%s] is valid.\n",
                                   state->certs[state->iter]);
-        ret = get_ssh_key_from_cert(state->keys,
-                                    state->bin_certs[state->iter].data,
-                                    state->bin_certs[state->iter].length,
-                                    &state->keys[state->iter].data,
-                                    &state->keys[state->iter].length);
+        ret = get_ssh_key_from_derb64(state->keys,
+                                      state->certs[state->iter],
+                                      &state->keys[state->iter].data,
+                                      &state->keys[state->iter].length);
         if (ret == EOK) {
             state->valid_keys++;
         } else {
