@@ -45,7 +45,7 @@ int debug_to_file = 0;
 int debug_to_stderr = 0;
 enum sss_logger_t sss_logger;
 const char *debug_log_file = "sssd";
-FILE *debug_file = NULL;
+static FILE *debug_file;
 
 const char *sss_logger_str[] = {
         [STDERR_LOGGER] = "stderr",
@@ -170,6 +170,9 @@ int debug_convert_old_level(int old_level)
     if (old_level >= 9)
         new_level |= SSSDBG_TRACE_ALL | SSSDBG_BE_FO;
 
+    if (old_level >= 10)
+        new_level |= SSSDBG_TRACE_LDB;
+
     return new_level;
 }
 
@@ -270,8 +273,6 @@ void sss_vdebug_fn(const char *file,
 {
     struct timeval tv;
     struct tm *tm;
-    char datetime[20];
-    int year;
 
 #ifdef WITH_JOURNALD
     errno_t ret;
@@ -300,25 +301,17 @@ void sss_vdebug_fn(const char *file,
     if (debug_timestamps) {
         gettimeofday(&tv, NULL);
         tm = localtime(&tv.tv_sec);
-        year = tm->tm_year + 1900;
-        /* get date time without year */
-        memcpy(datetime, ctime(&tv.tv_sec), 19);
-        datetime[19] = '\0';
+        debug_printf("(%d-%02d-%02d %2d:%02d:%02d",
+                     tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+                     tm->tm_hour, tm->tm_min, tm->tm_sec);
         if (debug_microseconds) {
-            debug_printf("(%s:%.6ld %d) [%s] [%s] (%#.4x): ",
-                         datetime, tv.tv_usec,
-                         year, debug_prg_name,
-                         function, level);
-        } else {
-            debug_printf("(%s %d) [%s] [%s] (%#.4x): ",
-                         datetime, year,
-                         debug_prg_name, function, level);
+            debug_printf(":%.6ld", tv.tv_usec);
         }
-    } else {
-        debug_printf("[%s] [%s] (%#.4x): ",
-                     debug_prg_name, function, level);
+        debug_printf("): ");
     }
 
+    debug_printf("[%s] [%s] (%#.4x): ", debug_prg_name, function, level);
+
     debug_vprintf(format, ap);
     if (flags & APPEND_LINE_FEED) {
         debug_printf("\n");
@@ -355,7 +348,7 @@ void ldb_debug_messages(void *context, enum ldb_debug_level level,
         loglevel = SSSDBG_TRACE_FUNC;
         break;
     case LDB_DEBUG_TRACE:
-        loglevel = SSSDBG_TRACE_ALL;
+        loglevel = SSSDBG_TRACE_LDB;
         break;
     }
 
@@ -472,31 +465,33 @@ int rotate_debug_files(void)
 
     if (sss_logger != FILES_LOGGER) return EOK;
 
-    do {
-        error = 0;
-        ret = fclose(debug_file);
-        if (ret != 0) {
-            error = errno;
+    if (debug_file != NULL) {
+        do {
+            error = 0;
+            ret = fclose(debug_file);
+            if (ret != 0) {
+                error = errno;
+            }
+
+            /* Check for EINTR, which means we should retry
+             * because the system call was interrupted by a
+             * signal
+             */
+        } while (error == EINTR);
+
+        if (error != 0) {
+            /* Even if we were unable to close the debug log, we need to make
+             * sure that we open up a new one. Log rotation will remove the
+             * current file, so all debug messages will be disappearing.
+             *
+             * We should write an error to the syslog warning of the resource
+             * leak and then proceed with opening the new file.
+             */
+            sss_log(SSS_LOG_ALERT, "Could not close debug file [%s]. [%d][%s]\n",
+                                   debug_log_file, error, strerror(error));
+            sss_log(SSS_LOG_ALERT, "Attempting to open new file anyway. "
+                                   "Be aware that this is a resource leak\n");
         }
-
-        /* Check for EINTR, which means we should retry
-         * because the system call was interrupted by a
-         * signal
-         */
-    } while (error == EINTR);
-
-    if (error != 0) {
-        /* Even if we were unable to close the debug log, we need to make
-         * sure that we open up a new one. Log rotation will remove the
-         * current file, so all debug messages will be disappearing.
-         *
-         * We should write an error to the syslog warning of the resource
-         * leak and then proceed with opening the new file.
-         */
-        sss_log(SSS_LOG_ALERT, "Could not close debug file [%s]. [%d][%s]\n",
-                               debug_log_file, error, strerror(error));
-        sss_log(SSS_LOG_ALERT, "Attempting to open new file anyway. "
-                               "Be aware that this is a resource leak\n");
     }
 
     debug_file = NULL;
