@@ -103,6 +103,8 @@ extern int dbus_activated;
 #define FLAGS_GEN_CONF 0x0008
 #define FLAGS_NO_WATCHDOG 0x0010
 
+#define SSS_WATCHDOG_EXIT_CODE 70 /* to match EX_SOFTWARE in sysexits.h */
+
 #define PIPE_INIT { -1, -1 }
 
 #define PIPE_FD_CLOSE(fd) do {      \
@@ -183,6 +185,7 @@ struct main_context {
 errno_t server_common_rotate_logs(struct confdb_ctx *confdb,
                                   const char *conf_entry);
 int die_if_parent_died(void);
+int check_pidfile(const char *file);
 int pidfile(const char *file);
 int server_setup(const char *name, int flags,
                  uid_t uid, gid_t gid,
@@ -366,6 +369,11 @@ errno_t check_and_open_readonly(const char *filename, int *fd,
 bool check_ipv4_addr(struct in_addr *addr, uint8_t check);
 bool check_ipv6_addr(struct in6_addr *addr, uint8_t check);
 
+/* Returns the canonical form of an IPv4 or IPv6 address */
+errno_t sss_canonicalize_ip_address(TALLOC_CTX *mem_ctx,
+                                    const char *address,
+                                    char **canonical_address);
+
 const char * const * get_known_services(void);
 
 errno_t sss_user_by_name_or_uid(const char *input, uid_t *_uid, gid_t *_gid);
@@ -486,6 +494,9 @@ remove_ipv6_brackets(char *ipv6addr);
 errno_t add_string_to_list(TALLOC_CTX *mem_ctx, const char *string,
                            char ***list_p);
 
+errno_t del_string_from_list(const char *string,
+                             char ***list_p, bool case_sensitive);
+
 bool string_in_list(const char *string, char **list, bool case_sensitive);
 
 int domain_to_basedn(TALLOC_CTX *memctx, const char *domain, char **basedn);
